use std::{path::PathBuf, str::FromStr};

use serde::{Deserialize, Serialize};

use crate::utils::command::is_command_valid;

use super::{AbstractLinterConfiguration, LinterStatus};

#[derive(Default, Debug, Serialize, Deserialize)]
pub struct ModelsimConfiguration {
    pub language_id: String,
    pub linter: ModelsimLinter
}


#[derive(Debug, Serialize, Deserialize)]
pub struct ModelsimLinter {
    pub name: String,
    /// 目前是否启动
    pub enabled: bool,
    pub path: String,
    pub args: Vec<String>,
}


impl Default for ModelsimLinter {
    fn default() -> Self {
        Self {
            name: "modelsim".to_string(),
            enabled: false,
            path: "Modelsim-verilog-syntax".to_string(),
            args: Vec::new(),
        }
    }
}


impl AbstractLinterConfiguration for ModelsimConfiguration {
    fn new(language_id: &str) -> Self {
        ModelsimConfiguration {
            language_id: language_id.to_string(),
            linter: ModelsimLinter::default()
        }
    }
    
    fn provide_diagnostics(&self) {

    }

    fn get_windows_extension(&self) -> &str {
        "exe"
    }

    fn get_linter_path(&self) -> &str {
        &self.linter.path
    }

    fn linter_status(&self) -> LinterStatus {
        match self.language_id.as_str() {
            "verilog" => {
                let invoke_name = self.get_invoke_name("vlog");
                let available = is_command_valid(&invoke_name);
                LinterStatus {
                    tool_name: "vlog".to_string(),
                    available,
                    invoke_name
                }
            }

            "systemverilog" => {
                let invoke_name = self.get_invoke_name("vlog");
                let available = is_command_valid(&invoke_name);
                LinterStatus {
                    tool_name: "vlog".to_string(),
                    available,
                    invoke_name
                }
            }

            "vhdl" => {
                let invoke_name = self.get_invoke_name("vcom");
                let available = is_command_valid(&invoke_name);
                LinterStatus {
                    tool_name: "vcom".to_string(),
                    available,
                    invoke_name
                }
            }

            _ => {
                LinterStatus::default()
            }
        }
    }
}