use serde::{Deserialize, Serialize};

use crate::utils::command::is_command_valid;

use super::{AbstractLinterConfiguration, LinterStatus};

#[derive(Debug, Default, Serialize, Deserialize)]
pub struct VivadoConfiguration {
    pub language_id: String,
    pub linter: VivadoLinter,
}

#[derive(Debug, Serialize, Deserialize)]
pub struct VivadoLinter {
    pub name: String,
    /// 目前是否启动
    pub enabled: bool,
    pub path: String,
    pub args: Vec<String>,
}

impl Default for VivadoLinter {
    fn default() -> Self {
        Self {
            name: "vivado".to_string(),
            enabled: false,
            path: "Vivado".to_string(),
            args: vec![
                "--lint-only".to_string(),
                "--sv".to_string(),
                "-Wall".to_string(),
            ],
        }
    }
}

impl AbstractLinterConfiguration for VivadoConfiguration {
    fn new(language_id: &str) -> Self {
        VivadoConfiguration {
            language_id: language_id.to_string(),
            linter: VivadoLinter::default()
        }
    }

    fn provide_diagnostics(&self) {

    }

    fn get_linter_path(&self) -> &str {
        &self.linter.path
    }

    fn get_windows_extension(&self) -> &str {
        "exe"
    }

    fn linter_status(&self) -> LinterStatus {
        match self.language_id.as_str() {
            "verilog" => {
                let invoke_name = self.get_invoke_name("xvlog");
                let available = is_command_valid(&invoke_name);
                LinterStatus {
                    tool_name: "xvlog".to_string(),
                    available,
                    invoke_name
                }
            }

            "systemverilog" => {
                let invoke_name = self.get_invoke_name("xvlog");
                let available = is_command_valid(&invoke_name);
                LinterStatus {
                    tool_name: "xvlog".to_string(),
                    available,
                    invoke_name
                }
            }

            "vhdl" => {
                let invoke_name = self.get_invoke_name("xvhdl");
                let available = is_command_valid(&invoke_name);
                LinterStatus {
                    tool_name: "xvhdl".to_string(),
                    available,
                    invoke_name
                }
            }

            _ => {
                LinterStatus::default()
            }
        }
    }
}